#ifndef CANVERSION_H 
#define CANVERSION_H 
#define CAN_MAJOR_VERSION 6 
#define CAN_MINOR_VERSION 1 
#endif 
/******************************************************************************
//
// FILE:		can.h 
//
// DESCRIPTION: The CAN interface.
//
// DOCUMENTS:	-
//
******************************************************************************/

/******************************************************************************
// NESTING HEADER
******************************************************************************/
#ifndef CAN_H
#define CAN_H


#if defined(_DEBUG) && !defined(NDEBUG)

#define CANAPI static

#if !defined(CANDLL)
#define CANAPIDBG __declspec(dllimport)
#else
#define CANAPIDBG __declspec(dllexport)
#endif

#else

#define CANAPIDBG static

#if !defined(CANDLL)
#define CANAPI __declspec(dllimport)
#else
#define CANAPI __declspec(dllexport)
#endif

#endif


/******************************************************************************
// INCLUDE FILES
******************************************************************************/

#pragma warning( push, 3 )

#if !defined(_MFC)

#include <windows.h>

#else

#include <afx.h>

#endif

#include <winioctl.h>

#pragma warning( pop )

#include "candrv_public.h"

/******************************************************************************
// MANIFEST CONSTANTS
******************************************************************************/

/******************************************************************************
// TYPES
******************************************************************************/

struct _CanHandle;
typedef struct _CanHandle *CanHandle;

typedef struct _CanStatistics
{
	ULONG	rxMsgCntr;		// The number CAN messages received so far.
	ULONG	rxDataCntr;		// The number of data bytes received so far.
	ULONG	txMsgCntr;		// The number CAN messages transmitted so far.
	ULONG	txDataCntr;		// The number of data bytes transmitted so far.
	ULONG	hwOvrnCntr;		// The number of times that the CAN controller has
							// detected a receive overrun.
	ULONG	busWarnCntr;	// The number of times that the CAN controller has
							// experienced a bus warning condition.
	ULONG	busOffCntr;		// The number of times that the CAN controller has
							// entered the bus off state.
	ULONG	appOvrnCntr;	// The maximum number of overruns that have
							// occurred for a client application. If this
							// occurs then the client application needs to be
							// optimized.
	ULONG	rxFifoOvrnCntr;	// The number of times that a overrun has occurred
							// in the type-ahead receive fifo. If this occurs
							// then the size of the type-ahead receive fifo
							// needs to be increased, see configuration value
							// "RxFifoSize".
	ULONG	rxFifoMax;		// The maximum number of messages that the
							// type-ahead receive fifo have contained.
} CanStatistics;

/******************************************************************************
// FUNCTION PROTOTYPES 
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
//
// FUNCTION:	CanOpenEx
//
// DESCRIPTION:	Opens the CAN interface with Super User privilege.
//				Only one user in the system may have Super User privilege
//				for a specific interface.
//				The caller may send messages with any CAN id.
//				The caller may receive messages from the interface without
//				"stealing" messages from others that have called CanOpen.
//
// PARAMETERS:	pNetName	The CAN net name, CAN1-CANn.
//							Either ANSI or UNICODE string , depending on if
//							_UNICODE and UNICODE is defined or not.
//
// RETURNS:		A handle to the opened interface or NULL if operation failed.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//				If the operation failed because of that another user
//				already has called CanOpenEx then GetLastError will return
//				ERROR_SHARING_VIOLATION.
//
******************************************************************************/
CANAPI CanHandle
CanOpenEx(
	LPCTSTR pNetName
	);

/******************************************************************************
//
// FUNCTION:	CanOpen
//
// DESCRIPTION:	Opens the CAN interface with Normal User privilege.
//				There is no restriction on the number of users with Normal User
//				privilege for a specific interface.
//				The caller may only send messages with CAN id's within the
//				configured Normal User range.
//				The caller may receive messages from the interface without
//				"stealing" messages from others that have called CanOpen.
//
// PARAMETERS:	pNetName	The CAN net name, CAN1-CANn.
//							Either ANSI or UNICODE string , depending on if
//							_UNICODE and UNICODE is defined or not.
//
// RETURNS:		A handle to the opened interface or NULL if operation failed.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//
******************************************************************************/
CANAPI CanHandle
CanOpen(
	LPCTSTR pNetName
	);

/******************************************************************************
//
// FUNCTION:	CanClose
//
// DESCRIPTION:	Closes the CAN interface.
//
// PARAMETERS:	hInterface	A handle to the opened interface.
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//
******************************************************************************/
CANAPI BOOL
CanClose(
	CanHandle hInterface
	);

/******************************************************************************
//
// FUNCTION:	CanSend
//
// DESCRIPTION:	Sends a message on the CAN interface.
//
// PARAMETERS:	hInterface	A handle to the opened interface.
//				pCanMsg		A pointer to the message to send.
//				dataLength	The number of data bytes to send.
//				bRtr		Should be set to TRUE if message is to be sent as a
//							remote frame.
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//				If the operation failed because of the interface was opened
//				with CanOpen and the CAN id of the message is outside the
//				Normal User range then GetLastError will return 
//				ERROR_ACCESS_DENIED.
//
******************************************************************************/
CANAPI BOOL
CanSend(
	CanHandle hInterface,
	CanMsg *pCanMsg,
	DWORD dataLength,
	BOOL bRtr
	);

/******************************************************************************
//
// FUNCTION:	CanSendEx
//
// DESCRIPTION:	Sends a standard or extended frame on the CAN interface.
//
// PARAMETERS:	hInterface	See "CanSend".
//				pCanMsg		See "CanSend".
//				dataLength	See "CanSend".
//				bRtr		See "CanSend".
//				frameType	Should be set to:
//							CAN_FRAME_STANDARD if message is to be sent as a 
//							standard frame.
//							CAN_FRAME_EXTENDED if message is to be sent as an
//							extended frame.
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//				If the operation failed because of the interface was opened
//				with CanOpen and the CAN id of the message is outside the
//				Normal User range then GetLastError will return 
//				ERROR_ACCESS_DENIED.
//				If the operation failed because the frame type selected by
//				frameType is not supported by the CAN controller or not
//				configured for the driver then "GetLastError" will return
//				ERROR_NOT_SUPPORTED.
//
******************************************************************************/
CANAPI BOOL
CanSendEx(
	CanHandle hInterface,
	CanMsg *pCanMsg,
	DWORD dataLength,
	BOOL bRtr,
	CanFrameType frameType
	);

/******************************************************************************
//
// FUNCTION:	CanReceive
//
// DESCRIPTION:	Receives a message from the CAN interface.
//
// PARAMETERS:	hInterface		A handle to the opened interface.
//				pCanMsg			A pointer to the received message.
//				pDataLength		A pointer to the number of data bytes
//								received.
//				pCanMsgSel		A pointer to an array specifying a selection
//								of which messages that should be received.
//								The first element, (pCanMsgSel[0]), should
//								specify the number of CAN message IDs in the
//								array.
//								If pCanMsgSel[0] is positive then
//								any of IDs in the array will be received.
//								If pCanMsgSel[0] is negative then
//								any of IDs that is not in the array will be
//								received.
//								If any message is reqested then NULL should
//								be supplied.
//				milliseconds	Specifies the time-out interval, in
//								milliseconds.
//								The function returns if the interval elapses,
//								even if no messages are received.
//								If milliseconds is zero, the function checks
//								if there are any messages and returns
//								immediately.
//								If milliseconds is INFINITE, the function
//								does not return until a message is received.
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//				If the operation failed because of that the time-out interval
//				expired then GetLastError will return ERROR_TIMEOUT.
//
******************************************************************************/
CANAPI BOOL
CanReceive(
	CanHandle hInterface,
	CanMsg *pCanMsg,
	LPDWORD pDataLength,
	CanMsgId *pCanMsgSel,
	DWORD milliseconds
	);

/******************************************************************************
//
// FUNCTION:	CanReceiveEx
//
// DESCRIPTION:	Receives a standard or extended frame from the CAN interface.
//
// PARAMETERS:	hInterface		See "CanReceive".
//				pCanMsg			See "CanReceive".
//				pDataLength		See "CanReceive".
//				pCanMsgSel		See "CanReceive".
//				frameTypeSel	Specifies which frame types that should be
//								received.
//								If frameTypeSel is set to CAN_FRAME_STANDARD
//								then only standard frames matching pCanMsgSel
//								is received.
//								If frameTypeSel is set to CAN_FRAME_EXTENDED
//								then only extended frames matching pCanMsgSel
//								is received.
//								If frameTypeSel is set to
//								CAN_FRAME_STANDARD | CAN_FRAME_EXTENDED then
//								either standard or extended frames matching
//								pCanMsgSel is received.
//								If frameTypeSel is set to any of the above, 
//								in combination with CAN_FRAME_REMOTE_ONLY,
//								then only remote frames will be received.
//				pFrameType		A pointer to the frame type of the received
//								message.
//								CAN_FRAME_STANDARD or CAN_FRAME_EXTENDED if a
//								standard or an extended frame is received.
//								CAN_FRAME_REMOTE if the frame is remote.
//				milliseconds	See "CanReceive".
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//				If the operation failed because of that the time-out interval
//				expired then GetLastError will return ERROR_TIMEOUT.
//				If the operation failed because the frame type selected by
//				frameTypeSel is not supported by the CAN controller or not
//				configured for the driver then "GetLastError" will return
//				ERROR_NOT_SUPPORTED.
//
******************************************************************************/
CANAPI BOOL
CanReceiveEx(
	CanHandle hInterface,
	CanMsg *pCanMsg,
	LPDWORD pDataLength,
	CanMsgId *pCanMsgSel,
	CanFrameType frameTypeSel,
	CanFrameType *pFrameType,
	DWORD milliseconds
	);

/******************************************************************************
//
// FUNCTION:	CanEnableRemoteFrame
//
// DESCRIPTION:	Enables reception of the specified remote frame.
//				This function is only needed/supported by hardware that uses
//				82527 alike message objects.
//				For other hardware this function will fail.
//
// PARAMETERS:	hInterface	A handle to the opened interface.
//				id			The CAN message identifier of the remote frame
//							to enable.
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//				If the operation failed because the interface was opened with
//				CanOpen and the CAN id of the message is outside the Normal
//				User range then GetLastError will return ERROR_ACCESS_DENIED.
//				If the operation failed because there is no more room for more
//				remote replies then GetLastError will return
//				ERROR_NO_MORE_ITEMS.
//				If the operation failed because the hardware doesn't
//				need/support the function then GetLastError will return
//				ERROR_NOT_SUPPORTED.
//
******************************************************************************/
CANAPI BOOL
CanEnableRemoteFrame(
	CanHandle hInterface,
	CanMsgId id
	);

/******************************************************************************
//
// FUNCTION:	CanDisableRemoteFrame
//
// DESCRIPTION:	Disables reception of the specified remote frame.
//				This function is only needed/supported by hardware that uses
//				82527 alike message objects.
//				For other hardware this function will fail.
//
// PARAMETERS:	hInterface	A handle to the opened interface.
//				id			The CAN message identifier of the remote frame
//							to disable.
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//				If the operation failed because the remote frame specified
//				by id has never been enabled, then GetLastError will
//				return ERROR_NOT_FOUND.
//				If the operation failed because the hardware doesn't
//				need/support the function then GetLastError will return
//				ERROR_NOT_SUPPORTED.
//
******************************************************************************/
CANAPI BOOL
CanDisableRemoteFrame(
	CanHandle hInterface,
	CanMsgId id
	);

/******************************************************************************
//
// FUNCTION:	CanEnumRemoteFrame
//
// DESCRIPTION:	Enumerates the enabled remote frames.
//				This function is only needed/supported by hardware that uses
//				82527 alike message objects.
//				For other hardware this function will fail.
//
// PARAMETERS:	hInterface	A handle to the opened interface.
//				index		Specifies the index of the subkey to retrieve.
//							This parameter should be zero for the first call to
//							the CanEnumRemoteReply function and then, as long
//							as ERROR_SUCCESS is returned, incremented for
//							subsequent calls until ERROR_NO_MORE_ITEMS are
//							returned.
//							returned and GetLastError returns . 
//				pCanMsgId	A pointer to the message id of the remote
//							frame .
//							
//				pFrameType	A pointer to the frame type of the remote
//							frame.
//
// RETURNS:		ERROR_SUCCESS or some error code.
//				If operation failed because there are no more remote frame
//				replies then ERROR_NO_MORE_ITEMS is returned.
//
******************************************************************************/
CANAPI DWORD
CanEnumRemoteFrame(
	CanHandle hInterface,
	ULONG index,
	CanMsgId	*pId,	
	CanFrameType *pFrameType
	);





/******************************************************************************
//
// FUNCTION:	CanAddRemoteReply
//
// DESCRIPTION:	Obsolete - declaration only for previous version.
//
// PARAMETERS:	hInterface	
//				pCanMsg		
//				dataLength	
//
// RETURNS:		
//
******************************************************************************/
CANAPI BOOL
CanAddRemoteReply(
	CanHandle hInterface,
	CanMsg *pCanMsg,
	DWORD dataLength
	);

/******************************************************************************
//
// FUNCTION:	CanRemoveRemoteReply
//
// DESCRIPTION:	Obsolete - declaration only for previous version.
//
// PARAMETERS:	hInterface	
//				id			
//
// RETURNS:		
//
******************************************************************************/
CANAPI BOOL
CanRemoveRemoteReply(
	CanHandle hInterface,
	CanMsgId id
	);

/******************************************************************************
//
// FUNCTION:	CanEnumRemoteReply
//
// DESCRIPTION:	Obsolete - declaration only for previous version.
//
// PARAMETERS:	hInterface	
//				index		
//				pCanMsg		
//				pDataLength	
//				pFrameType	
//
// RETURNS:		ERROR_NOT_SUPPORTED
//
******************************************************************************/
CANAPI DWORD
CanEnumRemoteReply(
	CanHandle hInterface,
	ULONG index,
	CanMsg *pCanMsg,
	LPDWORD pDataLength,
	CanFrameType *pFrameType
	);




/******************************************************************************
//
// FUNCTION:	CanGetStatistics
//
// DESCRIPTION:	Returns performance and error counters.
//
// PARAMETERS:	hInterface		A handle to the opened interface.
//				pStatistics		A pointer to the performance and error
//								counters.
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//
******************************************************************************/
CANAPI BOOL
CanGetStatistics(
	CanHandle hInterface,
	CanStatistics *pStatistics
	);

/******************************************************************************
//
// FUNCTION:	CanGetLastTimeStamp
//
// DESCRIPTION:	Returns the timestamp for the last received message.
//
// PARAMETERS:	hInterface		A handle to the opened interface.
//				pTimeStamp		A pointer to the time stamp.
//
// RETURNS:		TRUE if operation succeeded otherwise FALSE.
//				If operation failed then "GetLastError" can be used to get
//				information of the error.
//
******************************************************************************/
CANAPI BOOL
CanGetLastTimeStamp(
	CanHandle hInterface,
	CanTimeStamp *pTimeStamp
	);

/******************************************************************************
//
// FUNCTION:	CanGetDeviceHandle
//
// DESCRIPTION:	Returns the handle to the CAN device.
//				The handle is intended to be used in DeviceIoControl calls.
//
// PARAMETERS:	hInterface		A handle to the opened interface.
//
// RETURNS:		A handle to the CAN device or INVALID_HANDLE_VALUE if
//				operation failed.
//
******************************************************************************/
CANAPI HANDLE
CanGetDeviceHandle(
	CanHandle hInterface
	);

//
// Version checked CanOpen and CanOpenEx.
//
CANAPI CanHandle
CanOpenA(
	LPCSTR pNetName,
	ULONG majorVersion,
	ULONG minorVersion
	);

CANAPI CanHandle
CanOpenW(
	LPCWSTR pNetName,
	ULONG majorVersion,
	ULONG minorVersion
	);

CANAPI CanHandle
CanOpenExA(
	LPCSTR pNetName,
	ULONG majorVersion,
	ULONG minorVersion
	);

CANAPI CanHandle
CanOpenExW(
	LPCWSTR pNetName,
	ULONG majorVersion,
	ULONG minorVersion
	);

//
// Line and file insertions for debug build.
//
CANAPIDBG CanHandle
CanOpenADbg(
	LPCSTR pFile,
	long line,
	LPCSTR pNetName,
	ULONG majorVersion,
	ULONG minorVersion
	);

CANAPIDBG CanHandle
CanOpenWDbg(
	LPCSTR pFile,
	long line,
	LPCWSTR pNetName,
	ULONG majorVersion,
	ULONG minorVersion
	);

CANAPIDBG CanHandle
CanOpenExADbg(
	LPCSTR pFile,
	long line,
	LPCSTR pNetName,
	ULONG majorVersion,
	ULONG minorVersion
	);

CANAPIDBG CanHandle
CanOpenExWDbg(
	LPCSTR pFile,
	long line,
	LPCWSTR pNetName,
	ULONG majorVersion,
	ULONG minorVersion
	);

CANAPIDBG BOOL
CanCloseDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface
	);

CANAPIDBG BOOL
CanSendDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanMsg *pCanMsg,
	DWORD dataLength,
	BOOL bRtr
	);

CANAPIDBG BOOL
CanSendExDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanMsg *pCanMsg,
	DWORD dataLength,
	BOOL bRtr,
	CanFrameType frameType
	);

CANAPIDBG BOOL
CanReceiveDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanMsg *pCanMsg,
	LPDWORD pDataLength,
	CanMsgId *pCanMsgSel,
	DWORD milliseconds
	);

CANAPIDBG BOOL
CanReceiveExDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanMsg *pCanMsg,
	LPDWORD pDataLength,
	CanMsgId *pCanMsgSel,
	CanFrameType frameTypeSel,
	CanFrameType *pFrameType,
	DWORD milliseconds
	);

CANAPIDBG BOOL
CanEnableRemoteFrameDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanMsgId id
	);

CANAPIDBG BOOL
CanDisableRemoteFrameDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanMsgId id
	);

CANAPIDBG DWORD
CanEnumRemoteFrameDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	ULONG index,
	CanMsgId	*pId,	
	CanFrameType *pFrameType
	);

CANAPIDBG BOOL
CanGetStatisticsDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanStatistics *pStatistics
	);

CANAPIDBG BOOL
CanGetLastTimeStampDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanTimeStamp *pTimeStamp
	);

CANAPIDBG HANDLE
CanGetDeviceHandleDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface
	);


// Obsolete - declaration only for previous version.
CANAPIDBG BOOL
CanAddRemoteReplyDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanMsg *pCanMsg,
	DWORD dataLength
	);

//Obsolete - declaration only for previous version.
CANAPIDBG BOOL
CanRemoveRemoteReplyDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanMsgId id
	);

//Obsolete - declaration only for previous version.
CANAPIDBG DWORD
CanEnumRemoteReplyDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	ULONG index,
	CanMsg *pCanMsg,
	LPDWORD pDataLength,
	CanFrameType *pFrameType
	);


#if defined(_DEBUG) && !defined(NDEBUG)

#if defined(_UNICODE) || defined(UNICODE)

#define CanOpen( pNetName )		\
		CanOpenWDbg(			\
			__FILE__,			\
			__LINE__,			\
			pNetName,			\
			CAN_MAJOR_VERSION,	\
			CAN_MINOR_VERSION )

#define CanOpenEx( pNetName )	\
		CanOpenExWDbg(			\
			__FILE__,			\
			__LINE__,			\
			pNetName,			\
			CAN_MAJOR_VERSION,	\
			CAN_MINOR_VERSION )

#else	/* defined(_UNICODE) || defined(UNICODE) */

#define CanOpen( pNetName )		\
		CanOpenADbg(			\
			__FILE__,			\
			__LINE__,			\
			pNetName,			\
			CAN_MAJOR_VERSION,	\
			CAN_MINOR_VERSION )

#define CanOpenEx( pNetName )	\
		CanOpenExADbg(			\
			__FILE__,			\
			__LINE__,			\
			pNetName,			\
			CAN_MAJOR_VERSION,	\
			CAN_MINOR_VERSION )

#endif	/* defined(_UNICODE) || defined(UNICODE) */

#define CanClose( hInterface ) \
		CanCloseDbg( __FILE__, __LINE__, hInterface )

#define CanSend( hInterface, pCanMsg, dataLength, bRtr ) \
		CanSendDbg( __FILE__, __LINE__, hInterface, pCanMsg, dataLength, bRtr )

#define CanSendEx(			\
			hInterface,		\
			pCanMsg,		\
			dataLength,		\
			bRtr,			\
			frameType )		\
		CanSendExDbg(		\
			__FILE__,		\
			__LINE__,		\
			hInterface,		\
			pCanMsg,		\
			dataLength,		\
			bRtr,			\
			frameType )

#define CanReceive(			\
			hInterface,		\
			pCanMsg,		\
			pDataLength,	\
			pCanMsgSel,		\
			milliseconds )	\
		CanReceiveDbg(		\
			__FILE__,		\
			__LINE__,		\
			hInterface,		\
			pCanMsg,		\
			pDataLength,	\
			pCanMsgSel,		\
			milliseconds )

#define CanReceiveEx(		\
			hInterface,		\
			pCanMsg,		\
			pDataLength,	\
			pCanMsgSel,		\
			frameTypeSel,	\
			pFrameType,		\
			milliseconds )	\
		CanReceiveExDbg(	\
			__FILE__,		\
			__LINE__,		\
			hInterface,		\
			pCanMsg,		\
			pDataLength,	\
			pCanMsgSel,		\
			frameTypeSel,	\
			pFrameType,		\
			milliseconds )

#define CanEnableRemoteFrame( hInterface, id ) \
		CanEnableRemoteFrameDbg( __FILE__, __LINE__, hInterface, id )

#define CanDisableRemoteFrame( hInterface, id ) \
		CanDisableRemoteFrameDbg( __FILE__, __LINE__, hInterface, id )

#define CanEnumRemoteFrame(		\
			hInterface,			\
			index,				\
			pCanMsgId,			\
			pFrameType )		\
		CanEnumRemoteFrameDbg(	\
			__FILE__,			\
			__LINE__,			\
			hInterface,			\
			index,				\
			pCanMsgId,			\
			pFrameType )

// Obsolete function - only declaration
#define CanAddRemoteReply(		\
			hInterface,			\
			pCanMsg,			\
			dataLength )		\
		CanAddRemoteReplyDbg(	\
			__FILE__,			\
			__LINE__,			\
			hInterface,			\
			pCanMsg,			\
			dataLength )

// Obsolete function - only declaration
#define CanRemoveRemoteReply( hInterface, id ) \
		CanRemoveRemoteReplyDbg( __FILE__, __LINE__, hInterface, id )

// Obsolete function - only declaration
#define CanEnumRemoteReply(		\
			hInterface,			\
			index,				\
			pCanMsg,			\
			pDataLength,		\
			pFrameType )		\
		CanEnumRemoteReplyDbg(	\
			__FILE__,			\
			__LINE__,			\
			hInterface,			\
			index,				\
			pCanMsg,			\
			pDataLength,		\
			pFrameType )


#define CanGetStatistics( hInterface, pStatistics ) \
		CanGetStatisticsDbg( __FILE__, __LINE__, hInterface, pStatistics )

#define CanGetLastTimeStamp( hInterface, pTimeStamp ) \
		CanGetLastTimeStampDbg( __FILE__, __LINE__, hInterface, pTimeStamp )

#define CanGetDeviceHandle( hInterface ) \
		CanGetDeviceHandleDbg( __FILE__, __LINE__, hInterface )

#else	/* defined(_DEBUG) && !defined(NDEBUG) */

#if defined(_UNICODE) || defined(UNICODE)

#define CanOpen( pNetName ) \
		CanOpenW( pNetName, CAN_MAJOR_VERSION, CAN_MINOR_VERSION )

#define CanOpenEx( pNetName ) \
		CanOpenExW( pNetName, CAN_MAJOR_VERSION, CAN_MINOR_VERSION )

#else	/* defined(_UNICODE) || defined(UNICODE) */

#define CanOpen( pNetName ) \
		CanOpenA( pNetName, CAN_MAJOR_VERSION, CAN_MINOR_VERSION )

#define CanOpenEx( pNetName ) \
		CanOpenExA( pNetName, CAN_MAJOR_VERSION, CAN_MINOR_VERSION )

#endif	/* defined(_UNICODE) || defined(UNICODE) */

#endif	/* defined(_DEBUG) && !defined(NDEBUG) */

/******************************************************************************
// Functions used by can tools.
// should not be called from a normal application
******************************************************************************/

typedef ULONG CanInterfaceType;

#define CAN_INTERFACE_REAL	0x1
#define CAN_INTERFACE_SIM	0x2

typedef struct _CanProperties
{
	CanInterfaceType	interfaceTypes;	// Interface types			
	CanFrameType		frameTypes;		// Supported frametypes.
} CanProperties;

CANAPI BOOL
CanGetProperties(
	CanHandle hInterface,
	CanProperties *pProperties
	);

CANAPIDBG BOOL
CanGetPropertiesDbg(
	LPCSTR pFile,
	long line,
	CanHandle hInterface,
	CanProperties *pProperties
	);

#if defined(_DEBUG) && !defined(NDEBUG)

#define CanGetProperties( hInterface, pProperties ) \
	CanGetPropertiesDbg( __FILE__, __LINE__, hInterface, pProperties )

#endif	/* defined(_DEBUG) && !defined(NDEBUG) */

#ifdef __cplusplus
}
#endif

/******************************************************************************
// NESTING TRAILER 
******************************************************************************/
#endif
